require 'faraday'

module Airbrake
  class FaradaySender
    extend Forwardable
    
    def_delegators :@configuration, :secure, :protocol, :host, :port, 
       :http_open_timeout, :http_read_timeout
    
    class << self
      attr_accessor :faraday_middleware
    end
    
    def initialize(configuration)
      @configuration = configuration
    end

    # Override the send to airbrake method
    def send_to_airbrake(data)
      log :debug, "Sending request to #{url.to_s}:\n#{data}"

      response = begin
        log :info, "Starting post::: #{url.path}"
        connection.post(url.path, data) do |req|
          req.headers = Airbrake::HEADERS
        end
      rescue Faraday::Error::ClientError => e
        log :error, "Error contacting Airbrake server: #{e.message}"
        return
      end

      log :info, "Success: #{response.class}", response
      
      if response.respond_to?(:body)
        error_id = response.body.match(%r{<error-id[^>]*>(.*?)</error-id>})
        error_id[1] if error_id
      end
    end
    
  private
  
    # Define the farday connection
    def connection
      ssl = {}
      if secure
        ssl[:use_ssl]     = true
        ssl[:ca_file]     = OpenSSL::X509::DEFAULT_CERT_FILE if File.exist?(OpenSSL::X509::DEFAULT_CERT_FILE)
        ssl[:verify_mode] = OpenSSL::SSL::VERIFY_PEER
      else
        ssl[:use_ssl]     = false
      end
    
      Faraday.new("#{url.scheme}://#{url.host}", {
          :ssl => ssl,
          :request => {:open_timeout => http_open_timeout, :timeout => http_read_timeout}
      }) do |builder|
        builder.response :raise_error
        
        if self.class.faraday_middleware
          self.class.faraday_middleware.call(builder) 
        else
          builder.adapter Faraday.default_adapter
        end
      end
    end
    
    def url
      URI.parse("#{protocol}://#{host}:#{port}").merge(Airbrake::Sender::NOTICES_URI)
    end
    
    def log(level, message, response = nil)
      logger.send level, LOG_PREFIX + message if logger
      Airbrake.report_environment_info
      Airbrake.report_response_body(response.body) if response && response.respond_to?(:body)
    end

    def logger
      Airbrake.logger
    end
  end
end